//Base HLSL vertex shader with a few modifications
struct VertexShaderInput
{
    float4 vPosition : POSITION;
	float2 vTexcoord : TEXCOORD0;
    float4 vNormal : NORMAL;
    float4 vColor    : COLOR0;
};

//The output is important to note
//This is the information that will be passed into the fragment shader
struct VertexShaderOutput
{
    float4 vPosition : SV_POSITION;
    float4 vColor    : COLOR;
    float2 vTexcoord : TEXCOORD;
	//Pass in the normals
	float4 vNormal : NORMAL0;
	//Pass in the camera direction (as a normal)
	float4 vCamera : NORMAL1;
	//Pass in the depth of the vertex
	float vDepth : DEPTH;
};

VertexShaderOutput main(VertexShaderInput INPUT)
{
    VertexShaderOutput OUTPUT;

	//Set the camera to the proper direction
	OUTPUT.vCamera = -float4(gm_Matrices[MATRIX_WORLD_VIEW][2][0],
						gm_Matrices[MATRIX_WORLD_VIEW][2][1],
						gm_Matrices[MATRIX_WORLD_VIEW][2][2],
						gm_Matrices[MATRIX_WORLD_VIEW][2][3]);

	//Build the world view projection
    float4 matrixWVP = mul(gm_Matrices[MATRIX_WORLD_VIEW_PROJECTION], INPUT.vPosition);
	
	//Get the depth as a value between 0 and 1
	OUTPUT.vDepth = matrixWVP.z / matrixWVP.w;
	//Pass in the rest of the values
    OUTPUT.vPosition = matrixWVP;
    OUTPUT.vColor    = INPUT.vColor;
    OUTPUT.vTexcoord = INPUT.vTexcoord;
    OUTPUT.vNormal = INPUT.vNormal;

    return OUTPUT;
}