
//This needs to match up to the varyings in the vertex shader
varying vec4 o_vPosition;
varying vec4 o_vColor;
varying vec2 o_vTexcoord;
varying vec4 o_vNormal;
varying vec4 o_vCamera;
varying float o_vDepth;

//This is a texture that will be used for the lighting ramp
uniform sampler2D rampTexture;

//This is the direction for the light
uniform vec3 lightDirection;

//Here's where the magic happens
void main(void)
{
	//--Color
		//Dot values are changed so that they range from 0,1 instead of -1,1 
		//Get the dot value between the normal and the light direction
		float lightDot = (dot(o_vNormal.rgb,lightDirection) *.5) + .5;
		//Get the dot value between the normal and the camera
		float cameraDot = (dot(o_vNormal.rgb,o_vCamera.rgb) *.5) + .5;
		//Get the dot value relative to the camera's up (using a swizzle)
		float upDot = (dot(o_vNormal.rgb,o_vCamera.rbg) *.5) + .5;
		//How far along the ramp texture to go
		//Change the .35 to determin how much impact the light has on the shading vs the camera angle
		//(0=only light has an impact and 1=only the camera has an impact)
		float rampAmount = mix(lightDot,cameraDot,.35);
    
		//Get the actual texture
		vec4 diffuseTexture = texture2D(gm_BaseTexture, o_vTexcoord);	
		//Get the lighting color based on the ramp amount
		vec4 rampColor = texture2D(rampTexture,vec2(rampAmount,rampAmount));
	
		//Cheat lighting
		//Shade is based on the cameraDot. 1.4 is sort of the threshold. Higer=more shade
		float shade = clamp(cameraDot * 1.4, 0.0, 1.0);
		//Extra cheat lighting. Anything below .9 gets cut out.
		//Lower .9 for more extra light (the two values should always add up to 1)
		//For example: clamp((lightDot-.7),0,.3) ... clamp((lightDot-.4),0,.6)
		float extraLight = clamp((lightDot-.9),0.0,.1);
		//(I think adding a little bit of normal shading on top of the toon shading looks good)
		//If you don't want it then remove the "shade" and "extraLight" lines
		//And change the comments on on the "OUTPUT.outDiffuse.rgb = ..." line
	
		//The OUTPUT.outDiffuse will write to the 0th surface supplied in the draw event
		//Return the texture color multiplied by the ramp color and shade value and then adding in the extra light
		gl_FragData[0].rgb = (diffuseTexture.rgb * rampColor.rgb * shade) + extraLight;
		//To remove cheat lighting ^Comment this out^
		//And uncomment  v this line v
		//gl_FragData[0].rgb = (diffuseTexture.rgb * rampColor.rgb);
		
		//Set the alpha to the normal texture alpha
		gl_FragData[0].a = diffuseTexture.a;
	//--End Color
	
	//--Normal and depth
		//The gl_FragData[1] will write to the 1th surface supplied in the draw event
		
		//Higher depth resolution means closer pixels will stand apart better
		//But it also caps how far away something can be before the depths become indistinguishable
		float depthResolution = 250.0;
		
		//Red is set based on the dot value of the camera's pointing direction and the surface normal
		gl_FragData[1].r = cameraDot;
		//Green is set based on the dot value of the camera's up direction and the surface normal
		gl_FragData[1].g = upDot;
		//Blue is set based on the depth of the vertex
		gl_FragData[1].b = o_vDepth;
		//Alpha is set based on the depth of the vertex but it loops based on the depthResolution
		gl_FragData[1].a = mod(o_vDepth * depthResolution, 1.00);
	//--End normal and depth
}

